@@ -3,8 +3,8 @@
 /*
     This file is part of Magnum.
 
-    Copyright © 2010, 2011, 2012, 2013, 2014, 2015, 2016, 2017, 2018, 2019
-              Vladimír Vondruš <mosra@centrum.cz>
+    Copyright © 2010, 2011, 2012, 2013, 2014, 2015, 2016, 2017, 2018, 2019,
+                2020 Vladimír Vondruš <mosra@centrum.cz>
 
     Permission is hereby granted, free of charge, to any person obtaining a
     copy of this software and associated documentation files (the "Software"),
@@ -183,7 +183,8 @@ template<class T> class Matrix4: public Matrix4x4<T> {
          * @cpp Matrix4::scaling(Vector3::yScale(-1.0f)) @ce. @f[
          *      \boldsymbol{A} = \boldsymbol{I} - 2 \boldsymbol{NN}^T ~~~~~ \boldsymbol{N} = \begin{pmatrix} n_x \\ n_y \\ n_z \end{pmatrix}
          * @f]
-         * @see @ref Matrix3::reflection(), @ref Vector::isNormalized()
+         * @see @ref Matrix3::reflection(), @ref Vector::isNormalized(),
+         *      @ref reflect()
          */
         static Matrix4<T> reflection(const Vector3<T>& normal);
 
@@ -366,6 +367,7 @@ template<class T> class Matrix4: public Matrix4x4<T> {
          *      ahead
          * @param far           Distance to far clipping plane, positive is
          *      ahead
+         * @m_since{2019,10}
          *
          * If @p far is finite, the result is: @f[
          *      \boldsymbol{A} = \begin{pmatrix}
@@ -445,8 +447,8 @@ template<class T> class Matrix4: public Matrix4x4<T> {
         /** @copydoc Matrix::Matrix(ZeroInitT) */
         constexpr explicit Matrix4(ZeroInitT) noexcept: Matrix4x4<T>{ZeroInit} {}
 
-        /** @copydoc Matrix::Matrix(NoInitT) */
-        constexpr explicit Matrix4(NoInitT) noexcept: Matrix4x4<T>{NoInit} {}
+        /** @copydoc Matrix::Matrix(Magnum::NoInitT) */
+        constexpr explicit Matrix4(Magnum::NoInitT) noexcept: Matrix4x4<T>{Magnum::NoInit} {}
 
         /** @brief Construct from column vectors */
         constexpr /*implicit*/ Matrix4(const Vector4<T>& first, const Vector4<T>& second, const Vector4<T>& third, const Vector4<T>& fourth) noexcept: Matrix4x4<T>(first, second, third, fourth) {}
@@ -706,6 +708,14 @@ template<class T> class Matrix4: public Matrix4x4<T> {
          *      \end{pmatrix}
          * @f]
          *
+         * Note that the returned vector is sign-less and the signs are instead
+         * contained in @ref rotation() const / @ref rotationShear() const in
+         * order to ensure @f$ \boldsymbol{R} \boldsymbol{S} = \boldsymbol{M} @f$
+         * for @f$ \boldsymbol{R} @f$ and @f$ \boldsymbol{S} @f$ extracted out
+         * of @f$ \boldsymbol{M} @f$. The signs can be extracted for example by
+         * applying @ref Math::sign() on a @ref diagonal(), but keep in mind
+         * that the signs can be negative even for pure rotation matrices.
+         *
          * @see @ref scalingSquared(), @ref uniformScaling(),
          *      @ref rotation() const, @ref Matrix3::scaling() const
          */
@@ -799,6 +809,7 @@ template<class T> class Matrix4: public Matrix4x4<T> {
 
         /**
          * @brief Normal matrix
+         * @m_since{2019,10}
          *
          * Returns @ref comatrix() of the upper-left 3x3 part of the matrix.
          * Compared to the classic transformation @f$ (\boldsymbol{M}^{-1})^T @f$,
